using System;
using System.Data.OleDb;
using System.Globalization;

using CustomDependency.Business;
using Alachisoft.NCache.Runtime.Dependencies;
using Alachisoft.NCache.Runtime.Serialization;
using Alachisoft.NCache.Runtime.Serialization.IO;

namespace CustomDependency
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	[Serializable]
	public class Dependency : ExtensibleDependency, ICompactSerializable
	{
		private string _connString;
		private int _productID;

        public override bool Initialize() { return false; }

		public Dependency(int productID, string connString)
		{
			_connString = connString;
			_productID = productID;
		}

		void ICompactSerializable.Serialize(CompactWriter writer)
		{
			writer.Write(_connString);
			writer.Write(_productID);
		}

		void ICompactSerializable.Deserialize(CompactReader reader)
		{
			_connString = reader.ReadString();
			_productID = reader.ReadInt32();
		}

		internal bool DetermineExpiration()
		{
			if (GetAvailableUnits(_productID) < 100)
				return true;
			return false;
		}
 
	
		internal int GetAvailableUnits(int productID)
		{
			OleDbDataReader reader = null;
			OleDbConnection connection = new OleDbConnection(_connString);
            connection.Open();
			int availableUnits = -1;
			try
			{
				OleDbCommand cmd = connection.CreateCommand();
				cmd.CommandText = String.Format(CultureInfo.InvariantCulture, 
					"Select UnitsInStock From Products" +
					" where ProductID = {0}", productID);

				reader = cmd.ExecuteReader();
					
				if(reader.Read())
				{
					availableUnits = Convert.ToInt32(reader["UnitsInStock"].ToString());
				}
				reader.Close();
				return availableUnits;
			}
			catch(Exception)
			{
				return availableUnits;
			}
		}
	
		public override bool HasChanged
		{
			get
			{
				return DetermineExpiration();
			}
		}
	}
}
