using System;
using System.Data.SqlClient;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Windows.Forms;
using System.Threading;
using System.Data.OleDb;

using CustomDependency.Business;

namespace DatabaseUpdater
{
	public class DBUpdater : Form
	{
		
		private System.Windows.Forms.Button btnStartStop;
		private System.Windows.Forms.TextBox _txtLog;
		private System.Windows.Forms.Button btnDone;
		private System.Windows.Forms.GroupBox gbMain;
		private OleDbConnection _connection;
		private string			_connString;
		private string			_dsFilePath;

        #region delegates
        private delegate void logDelegate(int id, int value);
        #endregion


        /// <summary>
		/// thread to update the unit price in the database
		/// </summary>
		System.Threading.Thread updater;


		public DBUpdater()
		{
            _connString = System.Configuration.ConfigurationSettings.AppSettings["conn-string"].ToString();
            _connection = new OleDbConnection(_connString);
			_connection.Open();
			InitializeComponent();
		}

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.IContainer components = null;

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		/// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
		protected override void Dispose(bool disposing)
		{
			if (disposing && (components != null))
			{
				components.Dispose();
			}
			
			if (_connection != null)
				_connection.Close();

			if (updater != null)
				updater.Abort();

			base.Dispose(disposing);
		}

		#region Windows Form Designer generated code

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnStartStop = new System.Windows.Forms.Button();
			this._txtLog = new System.Windows.Forms.TextBox();
			this.btnDone = new System.Windows.Forms.Button();
			this.gbMain = new System.Windows.Forms.GroupBox();
			this.gbMain.SuspendLayout();
			this.SuspendLayout();
			// 
			// btnStartStop
			// 
			this.btnStartStop.BackColor = System.Drawing.SystemColors.Control;
			this.btnStartStop.Location = new System.Drawing.Point(173, 286);
			this.btnStartStop.Name = "btnStartStop";
			this.btnStartStop.Size = new System.Drawing.Size(63, 25);
			this.btnStartStop.TabIndex = 0;
			this.btnStartStop.Text = "&Start";
			this.btnStartStop.Click += new System.EventHandler(this.btnStartStop_Click);
			// 
			// _txtLog
			// 
			this._txtLog.Location = new System.Drawing.Point(5, 15);
			this._txtLog.Multiline = true;
			this._txtLog.Name = "_txtLog";
			this._txtLog.Size = new System.Drawing.Size(280, 245);
			this._txtLog.TabIndex = 2;
			this._txtLog.TabStop = false;
			// 
			// btnDone
			// 
			this.btnDone.BackColor = System.Drawing.SystemColors.Control;
			this.btnDone.Location = new System.Drawing.Point(242, 286);
			this.btnDone.Name = "btnDone";
			this.btnDone.Size = new System.Drawing.Size(63, 25);
			this.btnDone.TabIndex = 1;
			this.btnDone.Text = "&Close";
			this.btnDone.Click += new System.EventHandler(this.btnDone_Click);
			// 
			// gbMain
			// 
			this.gbMain.Controls.Add(this._txtLog);
			this.gbMain.Location = new System.Drawing.Point(10, 10);
			this.gbMain.Name = "gbMain";
			this.gbMain.Size = new System.Drawing.Size(295, 270);
			this.gbMain.TabIndex = 0;
			this.gbMain.TabStop = false;
			this.gbMain.Text = "Products Updation Log";
			// 
			// DBUpdater
			// 
			this.BackColor = System.Drawing.Color.WhiteSmoke;
			this.ClientSize = new System.Drawing.Size(312, 323);
			this.Controls.Add(this.gbMain);
			this.Controls.Add(this.btnDone);
			this.Controls.Add(this.btnStartStop);
			this.Location = new System.Drawing.Point(75, 75);
			this.MaximizeBox = false;
			this.Name = "DBUpdater";
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
			this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
			this.Text = "NorthWind Products Updater";
			this.gbMain.ResumeLayout(false);
			this.gbMain.PerformLayout();
			this.ResumeLayout(false);

		}

		#endregion


		#region >>      === Event Handlers ===      <<
		/// <summary>start/stop button event handler.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void btnStartStop_Click(object sender, EventArgs e)
		{
			if (updater == null)
			{
				// start the thread if it hasn't been started yet.
				updater = new System.Threading.Thread(new ThreadStart(Start));
				updater.Priority = System.Threading.ThreadPriority.BelowNormal;
				updater.Name = "updater";
				updater.Start();
				btnStartStop.Text = "&Stop";

				InitializeLog();
			}
			else if(updater != null)
			{
				// Interrupt the thread if it is running
				updater.Abort();
				updater = null;
				btnStartStop.Text = "&Start";
			}
		}


		private void btnDone_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		#endregion
        
		#region >>      === Thread Functions ===      <<


		/// <summary>method that updater thread runs
		/// infinately unless application is closed.
		/// this thread is only suspended or resumed by the appliation.
		/// </summary>
		private void Start()
		{
			while (true)
			{
				Random ran = new Random();

				int id = ran.Next(1, 15);    // select one row from first 20.
				int newStock = ran.Next(0, 200); // new price for the product

				UpdateUnitsInStock(id, newStock); // update the available stock
				System.Threading.Thread.Sleep(2000);
			}
		}

		/// <summary>updates the price in the database 
		/// randomly with random values
		/// </summary>
		/// <param name="id"></param>
		/// <param name="newPrice"></param>
		private void UpdateUnitsInStock(int id, int quantity)
		{
            lock (this)
            {
                try
                {
                    OleDbCommand cmd = _connection.CreateCommand();

                    cmd.CommandText = String.Format(CultureInfo.InvariantCulture,
                        "Update Products " +
                        "Set UnitsInStock = {0} " +
                        "Where ProductID = {1}",
                        quantity, id);

                    cmd.ExecuteNonQuery();
                    this.Invoke(new logDelegate(WriteToLog), id, quantity);
                }
                catch (Exception e)
                {
                    System.Diagnostics.Trace.WriteLine(e);
                }
            }
		}

		#endregion 

		#region  >>      === Utility Functions ===     <<

		public void InitializeLog()
		{
			//this._txtLog.Text = " Product Price Update Log" + "\r\n\r\n";
			this._txtLog.AppendText(String.Format("    {0,-12}\t{1,-24} ", "Product ID", "Available Stock Items") + "\r\n");
			this._txtLog.AppendText(String.Format("    {0,-15}\t{1,-24}\r\n", "----------------", "---------------------------------"));
		}
        
		//Write the result of updation in Log'
		public void WriteToLog(int productId, int UnitsInStock)
		{
			string entry;
			entry = String.Format("{0,12}\t\t{1,18}\r\n", productId, UnitsInStock);
			this._txtLog.AppendText(entry);
			this._txtLog.Focus();
			this._txtLog.ScrollToCaret();
		}

		//Write the result of updation in Log'
		public void WriteToLog(string text)
		{
			_txtLog.Text += text;
		}

		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			try
			{
				Application.Run(new DBUpdater());
			}
			catch(Exception e)
			{
				MessageBox.Show(e.Message, "Error");
			}
		}
	}
}